@@ -24,9 +24,10 @@
 
 #include "samplv1_sample.h"
 
-#include <QDomDocument>
-#include <QTextStream>
-#include <QFileInfo>
+#include "samplv1_sample.h"
+#include "samplv1_sched.h"
+
+#include "samplv1widget_config.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -63,6 +64,9 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.setupUi(this);
 
+	// Init sched notifier.
+	m_sched_notifier = new samplv1_sched_notifier(this);
+
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
 		m_params_ab[i] = samplv1_param::paramDefaultValue(samplv1::ParamIndex(i));
@@ -107,6 +111,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	shapes << tr("Pulse");
 	shapes << tr("Saw");
 	shapes << tr("Sine");
+	shapes << tr("Rand");
 	shapes << tr("Noise");
 
 	m_ui.Lfo1ShapeKnob->insertItems(0, shapes);
@@ -132,6 +137,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	states << tr("Off");
 	states << tr("On");
 
+	m_ui.Gen1ReverseKnob->insertItems(0, states);
 	m_ui.Gen1LoopKnob->insertItems(0, states);
 
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
@@ -209,6 +215,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// GEN1
 	setParamKnob(samplv1::GEN1_SAMPLE,  m_ui.Gen1SampleKnob);
+	setParamKnob(samplv1::GEN1_REVERSE, m_ui.Gen1ReverseKnob);
 	setParamKnob(samplv1::GEN1_LOOP,    m_ui.Gen1LoopKnob);
 	setParamKnob(samplv1::GEN1_OCTAVE,  m_ui.Gen1OctaveKnob);
 	setParamKnob(samplv1::GEN1_TUNING,  m_ui.Gen1TuningKnob);
@@ -468,7 +475,14 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	samplv1widget_config *pConfig = samplv1widget_config::getInstance();
+	if (pConfig)
+		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
+
 	// Menu actions
+	QObject::connect(m_ui.helpUseNativeDialogsAction,
+		SIGNAL(triggered(bool)),
+		SLOT(helpUseNativeDialogs(bool)));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -476,6 +490,11 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
+	// Special sample update notifications (eg. reverse)
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify()),
+		SLOT(updateSampleNotify()));
+
 	// Epilog.
 	// QWidget::adjustSize();
 
@@ -485,6 +504,13 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 }
 
 
+// Destructor.
+samplv1widget::~samplv1widget (void)
+{
+	delete m_sched_notifier;
+}
+
+
 // Param kbob (widget) map accesors.
 void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_knob *pKnob )
 {
@@ -569,6 +595,14 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 	++m_iUpdate;
 
 	switch (index) {
+#if 0//--updateSampleNotify();
+	case samplv1::GEN1_REVERSE: {
+		const bool bReverse = bool(fValue > 0.0f);
+		pSampl->setReverse(bReverse);
+		updateSample(pSampl->sample());
+		break;
+	}
+#endif
 	case samplv1::GEN1_LOOP: {
 		const bool bLoop = bool(fValue > 0.0f);
 		pSampl->setLoop(bLoop);
@@ -656,6 +690,28 @@ void samplv1widget::resetSwapParams (void)
 }
 
 
+// Initialize param values.
+void samplv1widget::updateParamValues (void)
+{
+	resetSwapParams();
+
+	samplv1 *pSampl = instance();
+
+	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
+		samplv1::ParamIndex index = samplv1::ParamIndex(i);
+		float fValue = samplv1_param::paramDefaultValue(index);
+		const float *pfParamPort
+			= (pSampl ? pSampl->paramPort(index) : NULL);
+		if (pfParamPort)
+			fValue = *pfParamPort;
+		setParamValue(index, fValue, true);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_params_ab[index] = fValue;
+	}
+}
+
+
 // Reset all param default values.
 void samplv1widget::resetParamValues (void)
 {
@@ -722,82 +778,26 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 	qDebug("samplv1widget::loadPreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	QFile file(sFilename);
-	if (!file.open(QIODevice::ReadOnly))
-		return;
-
-	static QHash<QString, samplv1::ParamIndex> s_hash;
-	if (s_hash.isEmpty()) {
-		for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-			samplv1::ParamIndex index = samplv1::ParamIndex(i);
-			s_hash.insert(samplv1_param::paramName(index), index);
-		}
-	}
-
 	clearSampleFile();
 
 	resetParamKnobs();
 	resetParamValues();
 
-	const QFileInfo fi(sFilename);
-	const QDir currentDir(QDir::current());
-	QDir::setCurrent(fi.absolutePath());
+	samplv1 *pSampl = instance();
+	if (pSampl == NULL)
+		return;
 
-	QDomDocument doc(SAMPLV1_TITLE);
-	if (doc.setContent(&file)) {
-		QDomElement ePreset = doc.documentElement();
-		if (ePreset.tagName() == "preset"
-			&& ePreset.attribute("name") == fi.completeBaseName()) {
-			for (QDomNode nChild = ePreset.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "samples") {
-					loadSamples(eChild);
-				}
-				else
-				if (eChild.tagName() == "params") {
-					for (QDomNode nParam = eChild.firstChild();
-							!nParam.isNull();
-								nParam = nParam.nextSibling()) {
-						QDomElement eParam = nParam.toElement();
-						if (eParam.isNull())
-							continue;
-						if (eParam.tagName() == "param") {
-							samplv1::ParamIndex index = samplv1::ParamIndex(
-								eParam.attribute("index").toULong());
-							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty()) {
-								if (!s_hash.contains(sName))
-									continue;
-								index = s_hash.value(sName);
-							}
-							float fValue = eParam.text().toFloat();
-						//--legacy support < 0.3.0.4 -- begin
-							if (index == samplv1::DEL1_BPM && fValue < 3.6f)
-								fValue *= 100.0f;
-						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue, true);
-							updateParam(index, fValue);
-							m_params_ab[index] = fValue;
-						}
-					}
-				}
-			}
-		}
-	}
+	samplv1_param::loadPreset(pSampl, sFilename);
 
-	file.close();
+	updateSample(pSampl->sample());
+	updateParamValues();
+
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
-	const QString& sPreset = fi.completeBaseName();
 	m_ui.Preset->setPreset(sPreset);
-
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
-
-	QDir::setCurrent(currentDir.absolutePath());
 }
 
 
@@ -806,138 +806,14 @@ void samplv1widget::savePreset ( const QString& sFilename )
 #ifdef CONFIG_DEBUG
 	qDebug("samplv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
-	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
 
-	const QFileInfo fi(sFilename);
-	const QDir currentDir(QDir::current());
-	QDir::setCurrent(fi.absolutePath());
+	samplv1_param::savePreset(instance(), sFilename);
 
-	QDomDocument doc(SAMPLV1_TITLE);
-	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", sPreset);
-	ePreset.setAttribute("version", SAMPLV1_VERSION);
-
-	QDomElement eSamples = doc.createElement("samples");
-	saveSamples(doc, eSamples);
-	ePreset.appendChild(eSamples);
-
-	QDomElement eParams = doc.createElement("params");
-	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
-		QDomElement eParam = doc.createElement("param");
-		samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", samplv1_param::paramName(index));
-		eParam.appendChild(
-			doc.createTextNode(QString::number(paramValue(index))));
-		eParams.appendChild(eParam);
-	}
-	ePreset.appendChild(eParams);
-	doc.appendChild(ePreset);
-
-	QFile file(sFilename);
-	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		QTextStream(&file) << doc.toString();
-		file.close();
-	}
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
-
-	QDir::setCurrent(currentDir.absolutePath());
-}
-
-
-// Sample serialization methods.
-void samplv1widget::loadSamples ( const QDomElement& eSamples )
-{
-	samplv1 *pSampl = instance();
-	if (pSampl == NULL)
-		return;
-
-	for (QDomNode nSample = eSamples.firstChild();
-			!nSample.isNull();
-				nSample = nSample.nextSibling()) {
-		QDomElement eSample = nSample.toElement();
-		if (eSample.isNull())
-			continue;
-		if (eSample.tagName() == "sample") {
-		//	int index = eSample.attribute("index").toInt();
-			QString sFilename;
-			uint32_t iLoopStart = 0;
-			uint32_t iLoopEnd = 0;
-			for (QDomNode nChild = eSample.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "filename") {
-				//	int index = eSample.attribute("index").toInt();
-					sFilename = eChild.text();
-				}
-				else
-				if (eChild.tagName() == "loop-start") {
-					iLoopStart = eChild.text().toULong();
-				}
-				else
-				if (eChild.tagName() == "loop-end") {
-					iLoopEnd = eChild.text().toULong();
-				}
-			}
-			// Legacy loader...
-			if (sFilename.isEmpty())
-				sFilename = eSample.text();
-			// Done it.
-			loadSampleFile(sFilename);
-			// Set actual sample loop points...
-			pSampl->setLoopRange(iLoopStart, iLoopEnd);
-			++m_iUpdate;
-			m_ui.Gen1Sample->setLoopStart(iLoopStart);
-			m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-			updateSampleLoop(pSampl->sample());
-			--m_iUpdate;
-		}
-	}
-
-	pSampl->reset();
-}
-
-
-void samplv1widget::saveSamples (
-	QDomDocument& doc, QDomElement& eSamples )
-{
-	samplv1 *pSampl = instance();
-	if (pSampl == NULL)
-		return;
-
-	const char *pszSampleFile = pSampl->sampleFile();
-	if (pszSampleFile == NULL)
-		return;
-
-	QDomElement eSample = doc.createElement("sample");
-	eSample.setAttribute("index", 0);
-	eSample.setAttribute("name", "GEN1_SAMPLE");
-
-	QDomElement eFilename = doc.createElement("filename");
-	eFilename.appendChild(doc.createTextNode(
-		QDir::current().relativeFilePath(
-			QString::fromUtf8(pszSampleFile))));
-	eSample.appendChild(eFilename);
-
-	uint32_t iLoopStart = pSampl->loopStart();
-	uint32_t iLoopEnd   = pSampl->loopEnd();
-	if (iLoopStart < iLoopEnd) {
-		QDomElement eLoopStart = doc.createElement("loop-start");
-		eLoopStart.appendChild(doc.createTextNode(
-			QString::number(iLoopStart)));
-		eSample.appendChild(eLoopStart);
-		QDomElement eLoopEnd = doc.createElement("loop-end");
-		eLoopEnd.appendChild(doc.createTextNode(
-			QString::number(iLoopEnd)));
-		eSample.appendChild(eLoopEnd);
-	}
-
-	eSamples.appendChild(eSample);
 }
 
 
@@ -968,6 +844,19 @@ void samplv1widget::openSample (void)
 }
 
 
+// Sample updater slot.
+void samplv1widget::updateSampleNotify (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("samplv1widget::updateSampleNotify()");
+#endif
+
+	samplv1 *pSampl = instance();
+	if (pSampl)
+		updateSample(pSampl->sample());
+}
+
+
 // Sample file reset.
 void samplv1widget::clearSampleFile (void)
 {
@@ -1175,6 +1064,16 @@ void samplv1widget::contextMenuRequest ( const QPoint& pos )
 
 
 // Menu actions.
+void samplv1widget::helpUseNativeDialogs ( bool bOn )
+{
+	samplv1widget_config *pConfig = samplv1widget_config::getInstance();
+	if (pConfig) {
+		pConfig->bUseNativeDialogs = bOn;
+		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+	}
+}
+
+
 void samplv1widget::helpAbout (void)
 {
 	// About...
@@ -1223,6 +1122,7 @@ void samplv1widget::helpAbout (void)
 	QMessageBox::about(this, tr("About") + " " SAMPLV1_TITLE, sText);
 }
 
+
 void samplv1widget::helpAboutQt (void)
 {
 	// About Qt...
